function output = grad_hess(func,x,del,varargin);
%------------------------------------------------------------------------------------
% PURPOSE: calculates the numerical gradient and hessian of a 
%          function specified by 'func' at the vector x
%------------------------------------------------------------------------------------
% USAGE: output    = grad_hess(func,x,varargin)
% where: func      = a matlab function 'func.m'
%        x         = the vector at which the gradient and hessian are to be evaluated
%        del       = norm of perturbations for calculation of derivatives
%        varargrin = extra variables to be passed through the subroutine
%------------------------------------------------------------------------------------
% RETURNS: an output 'structure'
% where: output.xeval    = x
%        output.delta    = derivative pertubation
%        output.gradient = numerical gradient
%        output.hessian  = numerical hessian
%------------------------------------------------------------------------------------
% Written by: Gregory Givens
%             MTSU Economics
%             16 January 2004
%------------------------------------------------------------------------------------



n = length(x);
grad = zeros(n,1);
hess = zeros(n,n);
delmat = del*eye(n);


func0 = eval([func '(x,varargin{:})']);

for i = 1:n;
    if size(x,1) > size(x,2);
        delx = delmat(:,i);
    else
        delx = delmat(i,:);
    end;
    grad0 = (eval([func '(x+delx,varargin{:})']) - eval([func '(x-delx,varargin{:})']))/(2*del);
    if abs(grad0) < 1e15;
        grad(i) = grad0;
    else
        grad(i) = NaN;
    end;
end;


for j = 1:n;
    if size(x,1) > size(x,2);
        delxj = delmat(:,j);
    else
        delxj = delmat(j,:);
    end;
    for i = 1:j;
        if size(x,1) > size(x,2);
            delxi = delmat(:,i);
        else
            delxi = delmat(i,:);
        end;
        if i == j;
            hess(i,j) = (eval([func '(x+delxi,varargin{:})']) + eval([func '(x-delxi,varargin{:})']) - 2*func0)/(del^2);
        else
            hess(i,j) = (eval([func '(x+delxi+delxj,varargin{:})']) - eval([func '(x+delxi-delxj,varargin{:})']) - ...
                eval([func '(x-delxi+delxj,varargin{:})']) + eval([func '(x-delxi-delxj,varargin{:})']))/(4*del^2);
        end;
    end;
end;
hess = hess + hess' - diag(diag(hess));


output.xeval = x;
output.delta = del;
output.gradient = grad;
output.hessian = hess;

